/******************************************************************************/
/*                                                                            */
/*  File: main.cpp                                                            */
/*  Author: bkenwright@xbdev.net                                              */
/*  URL: www.xbdev.net                                                        */
/*                                                                            */
/******************************************************************************/
/*
	swf parser/drawer tutorial/demo
*/
/******************************************************************************/

#define _WIN32_WINNT 0x0400

// Enable memory leak checking, you never know, little leaks sometimes sneak
// through, and this will let us know if one has
#define _CRTDBG_MAP_ALLOC
#include <stdlib.h>
#include <crtdbg.h>

#include <windows.h>
#include <shellapi.h>
#include <stdio.h>

#define ID_FILE_OPEN 9001

#pragma comment(lib, "D3d9.lib")
#pragma comment(lib, "D3dx9.lib")
#include <d3dx9.h>

#include "ttf.h"
#include "ttfdraw.h"

stTTF* g_ttf = NULL;

LPDIRECT3D9			g_pD3D			= NULL;
LPDIRECT3DDEVICE9	g_pD3DDevice	= NULL;
HWND				g_hWnd			= NULL;
float				g_timeStep		= 0.0f;
bool				g_leftButtonDown = false;
float				g_zoom = 1.0f;
D3DXVECTOR2			g_off = D3DXVECTOR2(0,0);
bool				g_keys[0xffff]	= {false};

//------------------------------------------------------------------------------

bool CreateDX()
{
	g_pD3D = Direct3DCreate9(D3D_SDK_VERSION);
	D3DDISPLAYMODE d3ddm;
	g_pD3D->GetAdapterDisplayMode(D3DADAPTER_DEFAULT, &d3ddm);


	D3DPRESENT_PARAMETERS d3dpp; 
	ZeroMemory(&d3dpp, sizeof(d3dpp));

	d3dpp.Windowed = TRUE;
	d3dpp.SwapEffect = D3DSWAPEFFECT_DISCARD;
	d3dpp.BackBufferFormat = d3ddm.Format;
	d3dpp.BackBufferCount=1;
	d3dpp.AutoDepthStencilFormat = D3DFMT_D16;
	d3dpp.EnableAutoDepthStencil = TRUE;

	HRESULT hr =
	g_pD3D->CreateDevice(	D3DADAPTER_DEFAULT, 
							D3DDEVTYPE_HAL,
							g_hWnd, 
							D3DCREATE_SOFTWARE_VERTEXPROCESSING, 
							&d3dpp, 
							&g_pD3DDevice);

	// If the graphics card doesn't support hardware acceleration
	if (hr!=S_OK)
	{
		g_pD3D->CreateDevice(	D3DADAPTER_DEFAULT, 
								D3DDEVTYPE_REF,
								g_hWnd, 
								D3DCREATE_SOFTWARE_VERTEXPROCESSING, 
								&d3dpp, 
								&g_pD3DDevice);
	}

	g_pD3DDevice->SetRenderState(D3DRS_LIGHTING, FALSE);

	g_pD3DDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, TRUE); 
	//g_pD3DDevice->SetRenderState(D3DRS_ALPHAFUNC, D3DCMP_GREATEREQUAL); 
	//g_pD3DDevice->SetRenderState(D3DRS_ALPHAREF, (DWORD)8); 
	//g_pD3DDevice->SetRenderState(D3DRS_ALPHATESTENABLE, TRUE);

	g_pD3DDevice->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);
	g_pD3DDevice->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);

	return true;
}// End CreateDX()

bool ReleaseDX()
{
	g_pD3DDevice->Release();
	g_pD3DDevice = NULL;

	g_pD3D->Release();
	g_pD3D = NULL;
	return true;
}// End ReleaseDX()

void GameLoopDX()
{
	if(!g_pD3DDevice)return;

	// We use a more accurate timing information, QueryPerformanceCounter
	// instead of GetTickCount() so we get smoother animations
	LARGE_INTEGER freq;
	QueryPerformanceFrequency(&freq);

	LARGE_INTEGER nTime;
	QueryPerformanceCounter(&nTime);

	static LARGE_INTEGER oTime = nTime;

	double delay = (double(nTime.QuadPart - oTime.QuadPart)/freq.QuadPart);
	oTime=nTime;

	g_timeStep = (float)delay;

	if (g_timeStep > 1.0f)
	{
		g_timeStep = 1.0f;
	}

	static float updateFPS = 0.0f;
	updateFPS += (float)delay;
	static float fps = 1.0f/(float)delay;
	if (updateFPS>1.0f)
	{
		updateFPS=0.0f;
		fps = 1.0f/(float)delay;
	}

	extern HWND g_hWnd;
	char title[256] = {0};
	sprintf(title, "TTF Font - Parser/Drawer - Press Keys - FPS:%.1f,  TimeStep:%.2f", fps, g_timeStep);
	SetWindowText(g_hWnd, title);

	g_pD3DDevice->BeginScene();

	g_pD3DDevice->Clear(	0, 
							NULL, 
							D3DCLEAR_TARGET|D3DCLEAR_ZBUFFER, 
							D3DCOLOR_XRGB(0xFF,0xFF,0xFF), 1.0f, 0 );


	DebugGlyfTrianglulate(g_pD3DDevice, g_ttf);

	g_pD3DDevice->EndScene();
	g_pD3DDevice->Present( NULL, NULL, NULL, NULL );

	{
		extern bool g_leftButtonDown; // main.cpp
		POINT pnt;
		GetCursorPos(&pnt);
		D3DXVECTOR2 cpos = D3DXVECTOR2((float)pnt.x, (float)pnt.y);
		static D3DXVECTOR2 oldPos = cpos;
		D3DXVECTOR2 delta = D3DXVECTOR2(0,0);
		if (g_leftButtonDown)
		{
			delta.x = oldPos.x - cpos.x;
			delta.y = oldPos.y - cpos.y;
		}
		else
		{
			oldPos = cpos;
		}
		g_off.x -= delta.x;
		g_off.y -= delta.y;

		oldPos = cpos;
	}
}// End GameLoopDX()

//------------------------------------------------------------------------------

long _stdcall MsgProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch (uMsg)
	{
		case WM_KEYDOWN:
		{
			switch( wParam )
			{
				case VK_ESCAPE:         // Exit on pressing Escape Key
					PostQuitMessage(0);
					break;
				case VK_TAB:	g_keys[VK_TAB]		=true;	break;
				case VK_RIGHT:	g_keys[VK_RIGHT]	=true;	break;
				case VK_LEFT:	g_keys[VK_LEFT]		=true;	break;
				case VK_RETURN:	g_keys[VK_RETURN]	=true;  break;
				case VK_CONTROL:g_keys[VK_CONTROL]	=true;  break;
				case VK_SHIFT:  g_keys[VK_SHIFT]	=true;  break;
				case VK_PAUSE:  g_keys[VK_PAUSE]	=true;  break;
				case VK_HOME:	g_keys[VK_HOME]		=true;  break;
				case VK_SPACE:	g_keys[VK_SPACE]	=true;  break;
				case VK_INSERT:	g_keys[VK_INSERT]	=true;  break;
			}
		}
		break;
		
		case WM_LBUTTONDOWN:
		{
			g_leftButtonDown = true;
		}
		break;

		case WM_LBUTTONUP:
		{
			g_leftButtonDown = false;
		}
		break;

		case WM_MOUSEWHEEL:
		{
			float val = float(GET_WHEEL_DELTA_WPARAM(wParam) / WHEEL_DELTA);

			const float zoomSpeed = 0.1f;
			g_zoom += val * zoomSpeed;
		}
		break;

		
		case WM_DROPFILES:
		{
			HDROP hDrop = (HDROP)wParam;
			//Get Length of Filename Dragged Dropped
			int numFiles = 
			DragQueryFile(hDrop,	// HDROP
						  -1,		// UINT
						  NULL,		// LPSTR
						  0);		// UINT

			if (numFiles<=0) break;

			//Get Length of Filename Dragged Dropped
			int strLen = 
			DragQueryFile(hDrop,
						  0,		// FileIndex
						  NULL,		
						  0);

			DBG_ASSERT(strLen<255);

			char fileName[256] = {0};
			//Get Filename Dragged Dropped
			DragQueryFile(hDrop,
						  0,		// FileIndex
						  fileName,	// FileName
						  strLen+1);// StringLen FileName

			if (strstr(&fileName[strLen-4], ".ttf")==0) break;

			delete g_ttf;
			g_ttf = new stTTF();
			ReadTTF(fileName, g_ttf);
		}
		break;

		case WM_DESTROY:
		{
			PostQuitMessage(0);
		}
		break;
	}

    return (long)DefWindowProc(hWnd, uMsg, wParam, lParam);
}// End MsgProc(..)


//------------------------------------------------------------------------------


int _stdcall WinMain(HINSTANCE i, HINSTANCE j, char* k, int) 
{
	// Turn on memory leak checking
	_CrtSetDbgFlag(_CrtSetDbgFlag(_CRTDBG_REPORT_FLAG)|_CRTDBG_LEAK_CHECK_DF);

	MSG msg;
	memset(&msg, 0, sizeof(msg));
	char szname[] = "TTF Parser/Drawer - Press keys to test various characters";
    WNDCLASSEX wc = {	sizeof(WNDCLASSEX), CS_CLASSDC, MsgProc, 0L, 0L, 
						GetModuleHandle(NULL), NULL, NULL, NULL, NULL,
						szname, NULL };
	wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
	RegisterClassEx( &wc );

	g_hWnd = CreateWindowEx(	WS_EX_APPWINDOW,
								szname, szname, 
								WS_OVERLAPPEDWINDOW,
								50, 50, 1500,900,
								//60, 1000, 100,100,
								//100, 100, 640,480,
								GetDesktopWindow(), NULL, wc.hInstance, NULL);


	DragAcceptFiles(g_hWnd, TRUE);
	CreateDX();
	IDirect3DDevice9* pd = g_pD3DDevice;



	/// Creation...

	g_ttf = new stTTF();
	//ReadTTF("times.ttf", g_ttf);

	ReadTTF("arial.ttf", g_ttf);

	//ReadTTF("distress.ttf", g_ttf);
	



	ShowWindow(g_hWnd, SW_SHOW);
	UpdateWindow(g_hWnd);

	while(1)
	{
		if (PeekMessage(&msg, NULL, 0, 0, PM_NOREMOVE))
		{
			if (!GetMessage(&msg, NULL, 0, 0))
				break;
			DispatchMessage(&msg);
        }
		else
		{
			GameLoopDX();
		}
	}

	// Releasing...

	delete g_ttf;
	g_ttf = NULL;

	ReleaseDX();

	_CrtDumpMemoryLeaks();

}// End WinMain(..)


//------------------------------------------------------------------------------

















