/******************************************************************************/
/*                                                                            */
/*  File: ttfdraw.h                                                           */
/*  Author: bkenwright@xbdev.net                                              */
/*  URL: www.xbdev.net                                                        */
/*                                                                            */
/******************************************************************************/
/*
	Draw TTF Font Information

	After reading in all the ttf information, I wrote a few quick set of functions 
	which are able to draw the points and splines.

	First is a simple draw spline algorithm, draws the outline, and a second
	part which is dirty and generates the triangles for the shape, so you can solid
	render the fonts.
	The trianglulation of the contours is even able to deal with wholes in the
	fonts and multiple contours.

*/
/******************************************************************************/

#pragma once


#include "..\DelaunayTriangulation\DelaunayTriangulation.h"


// Some globals....quick and dirty...for testing
const int MAX_CONTOURS = 200;
int g_numContours = 0;
int g_numVertsPerContour[MAX_CONTOURS] = {0};
D3DXVECTOR2 g_points[MAX_CONTOURS][1000];

void DrawSpline(IDirect3DDevice9* pDevice, D3DXVECTOR2 p0, D3DXVECTOR2 p1, D3DXVECTOR2 cntrPnt, DWORD col = 0xff00ff00)
{
	int contourIndx = g_numContours-1;
	DBG_ASSERT(contourIndx>=0);

	// p(t) = (1-t)^2 p0 + 2t(1-t)c0 + t^2 p2

	//int steps = 3;
	int steps = 6;
	for (int i=0; i<steps; i++)
	{
		float t = i/(float)steps;
		float ptx0 = (1-t)*(1-t)*p0.x + 2*t*(1-t)*cntrPnt.x + t*t*p1.x;
		float pty0 = (1-t)*(1-t)*p0.y + 2*t*(1-t)*cntrPnt.y + t*t*p1.y;

		t = (i+1)/(float)steps;
		float ptx1 = (1-t)*(1-t)*p0.x + 2*t*(1-t)*cntrPnt.x + t*t*p1.x;
		float pty1 = (1-t)*(1-t)*p0.y + 2*t*(1-t)*cntrPnt.y + t*t*p1.y;

		D3DXVECTOR2 pnts[] = { D3DXVECTOR2(ptx0,pty0), D3DXVECTOR2(ptx1,pty1) };
		DebugDrawLineList2D(pDevice, pnts, 2, col);


		DBG_ASSERT(contourIndx<MAX_CONTOURS-1);
		DBG_ASSERT(g_numVertsPerContour[contourIndx]<1000);
		g_points[contourIndx][ g_numVertsPerContour[contourIndx]++ ] = pnts[0];
	}

}

/******************************************************************************/

// ref: http://fontforge.sourceforge.net/bezier.html

void DebugDrawGlyf2(IDirect3DDevice9* pDevice, stTTF* ttf)
{
	//static int asciiKey = 67; // letter b
	//static int asciiKey = 82; // letter R
	static int asciiKey = 'l'; // letter R

	bool capsLockOn = (GetKeyState(0x14) & 0x01) ? false : true;

	for (int k=10; k<255; k++)
	{
		if (GetKeyState(k) & 0x80)
		{
			if ( (k>='0' && k<='9') )
			{
				asciiKey = k;
				break;
			}
			
			if ( (k>='A' && k<='Z') )
			{
				asciiKey = k;

				if (capsLockOn)
				{
					asciiKey = tolower(asciiKey);
				}
				break;
			}
		}
	}

	int asciiIndx = asciiKey;
	if (ttf->m_cmap.glyphIndexArray)
	{
		asciiIndx = ttf->m_cmap.glyphIndexArray[asciiKey];
	}

	stGLYFItem* glyfItem = &ttf->m_glyf.glyfItem[asciiIndx];

	stGlyfDescription* glyfDesc = &glyfItem->m_glyfDescription;

	float yDelta = (float)(glyfDesc->yMax - glyfDesc->yMin);
	float xDelta = (float)(glyfDesc->xMax - glyfDesc->xMin);

	if (glyfDesc->numberOfContours <=0 ) DBG_HALT; // Not supported yet! - Complex Shapes

	stGLYFSimple* glyfSimple = glyfItem->m_glyfSimple;
	DBG_ASSERT(glyfSimple);

	int numFlags = glyfSimple->numFlags;


	for (int c=0; c<glyfDesc->numberOfContours; c++)
	{
		g_numContours++;


		int startIndx = 0;
		if (c>0) startIndx = glyfSimple->endPtsOfContours[c-1] + 1;
		int endIndx = glyfSimple->endPtsOfContours[c] + 1;

		DBG_ASSERT( numFlags == glyfSimple->endPtsOfContours[ glyfDesc->numberOfContours-1 ]+1 );

		D3DXVECTOR2 ctrlPoints[3];
		D3DXVECTOR2 points[2];
		int numCtrlPoints = 0;
		int numPoints = 0;

		//for (int i=0; i<numFlags+1; i++)
		for (int i=startIndx; i<endIndx+1; i++)
		{
			int k = i;
			// Close the shape
			if (i==endIndx)
			{
				k=startIndx;
				uint8 flagStart = glyfSimple->flags[k];
				DBG_ASSERT(flagStart&0x1); // Should start on the curve
			}


			DBG_ASSERT(numPoints>=0		&& numPoints<2);
			DBG_ASSERT(numCtrlPoints>=0 && numCtrlPoints<2);

			uint8 flag = glyfSimple->flags[k];
			float xval = glyfSimple->xCoordinates[k] / xDelta;
			float yval = (1.0f - glyfSimple->yCoordinates[k] / yDelta) + 1.0f;
			D3DXVECTOR2 pnt = D3DXVECTOR2(xval, yval);

			D3DXVECTOR2 off = D3DXVECTOR2(0,-240);
			float s = 240.0f;
			pnt = pnt*s + off;

			if (flag & 0x1) // On Curve
			{
				points[numPoints] = pnt;
				numPoints++;
			}
			else // Control Point
			{
				ctrlPoints[numCtrlPoints] = pnt;
				numCtrlPoints++;
			}

			if (numPoints==2 || numCtrlPoints==2)
			{
				D3DXVECTOR2 p0, p1, c0;

				DBG_ASSERT(numCtrlPoints>=0 && numCtrlPoints>=0);

				DBG_ASSERT(!(numPoints==2 && numCtrlPoints==2));

				if (numCtrlPoints==2 && numPoints==1)
				{
					p0 = points[0];
					p1 = ctrlPoints[0] + (ctrlPoints[1] - ctrlPoints[0]) * 0.5f;
					c0 = ctrlPoints[0];

					points[0] = p1;
					ctrlPoints[0] = ctrlPoints[1];
					numCtrlPoints=1;
					DrawSpline(pDevice, p0, p1, c0, 0xffff0000);
				}
				else if (numPoints==2 && numCtrlPoints==0)
				{
					p0 = points[0];
					p1 = points[1];
					c0 = p0 + (p1-p0)*0.5f;

					points[0] = points[1];
					numPoints=1;
					DrawSpline(pDevice, p0, p1, c0, 0xff00ff00);
				}
				else if (numPoints==2 && numCtrlPoints==1)
				{
					p0 = points[0];
					p1 = points[1];
					c0 = ctrlPoints[0];

					points[0] = points[1];
					numPoints=1;
					numCtrlPoints=0;
					DrawSpline(pDevice, p0, p1, c0, 0xff0000ff);
				}
				else
				{
					DBG_HALT;
				}
			}
			DBG_ASSERT(numPoints>=0 && numPoints<2 && numCtrlPoints>=0 && numCtrlPoints<2);
		}
	}

}

/******************************************************************************/

void DebugGlyfTrianglulate(IDirect3DDevice9* pDevice, stTTF* ttf)
{
	static const int MAX_TRIS = 20000;
	static int g_triangleIndices[MAX_TRIS] = {0};
	static D3DXVECTOR2 verts[3000] = { D3DXVECTOR2(0,0) };
	static int numTris = 0;

	static bool once = true;

	//if (once)
	{
		g_numContours = 0;
		memset(g_numVertsPerContour, 0, sizeof(g_numVertsPerContour));

		DebugDrawGlyf2(pDevice, ttf);

#if 0
		dprintf("int numCountours = %d;\n", g_numContours);
		dprintf("int numVertsPerContour[] = {");
		for (int i=0; i<g_numContours; i++)
		{
			dprintf("%d,", g_numVertsPerContour[i]);
		}
		dprintf("};\n");

		dprintf("D3DXVECTOR2 verts[] = {");
		for (int c=0; c<g_numContours; c++)
		{
			int numVerts = g_numVertsPerContour[c];
			for (int i=0; i<numVerts; i++)
			{
				dprintf("	D3DXVECTOR2(%f, %f),\n", g_points[c][i].x, g_points[c][i].y);
			}
		}
		dprintf("};\n");
#endif

		
		int totalVerts = 0;
		for (int c=0; c<g_numContours; c++)
		{
			totalVerts += g_numVertsPerContour[c];
		}
		DBG_ASSERT(totalVerts<3000);
		
		//g_numContours = 7;

		int indx = 0;
		for (int c=0; c<g_numContours; c++)
		{
			int numVerts = g_numVertsPerContour[c];

			for (int i=0; i<numVerts; i++)
			{
				verts[indx].x = g_points[c][i].x;
				verts[indx].y = g_points[c][i].y;
				indx++;
			}
		}

		memset(g_triangleIndices, 0, sizeof(g_triangleIndices));
		numTris = 0;

		
#if 0
	static float triLimit = -1.0f;
	if( GetKeyState(VK_RIGHT)	& 0x80 )	triLimit+=0.9f; // '>' key
	if( GetKeyState(VK_LEFT)	& 0x80 )	triLimit-=0.9f; // '<' key
	triLimit = (triLimit<1.0f) ? 1.0f : triLimit;
#endif

		// *** Trianglulation Code is done here where we chop up
		// the contours and splines into shapes :)
		TriangulatePolygon(	g_numContours,
							g_numVertsPerContour,
							verts,
							MAX_TRIS,
							&numTris,
							g_triangleIndices);

		once = false;
	}

	extern D3DXVECTOR2 g_off;
	extern float g_zoom;
	float s = g_zoom;

	extern bool g_leftButtonDown; // main.cpp
	POINT pnt;
	GetCursorPos(&pnt);
	D3DXVECTOR2 cpos = D3DXVECTOR2((float)pnt.x, (float)pnt.y);
	static D3DXVECTOR2 oldPos = cpos;
	D3DXVECTOR2 delta = D3DXVECTOR2(0,0);
	if (g_leftButtonDown)
	{
		delta.x = oldPos.x - cpos.x;
		delta.y = oldPos.y - cpos.y;
	}
	else
	{
		oldPos = cpos;
	}
	g_off.x -= delta.x;
	g_off.y -= delta.y;

	oldPos = cpos;


	static bool drawLines = true;
	extern bool	g_keys[0xffff];
	if (g_keys[VK_TAB]) { g_keys[VK_TAB]=false; drawLines=!drawLines; }


	for (int i=0; i<numTris; i++)
	{
		int* triIndices = &g_triangleIndices[i * 3];

		int indx0 = triIndices[0];
		int indx2 = triIndices[1];
		int indx1 = triIndices[2];

		D3DXVECTOR2 p0 = verts[indx0];
		D3DXVECTOR2 p1 = verts[indx1];
		D3DXVECTOR2 p2 = verts[indx2];


		D3DXVECTOR2 pnts[] = {	g_off + p0*s, g_off + p1*s,
								g_off + p1*s, g_off + p2*s,
								g_off + p2*s, g_off + p0*s };

		if (drawLines)
		{
			DebugDrawLineList2D(pDevice, pnts, 6);
		}

		#if 1 // draw solid
			D3DXVECTOR2 triPnts[3] = {	p0*s + g_off, 
										p1*s + g_off, 
										p2*s + g_off};
			DebugDrawTri2D(pDevice, triPnts, 0x99ff0000);
		#endif
	}

}

/******************************************************************************/





